#ifndef RESOURCE_H
#define RESOURCE_H
#include <string>
#include "mime_types.hpp"
#include "reply.hpp"
#include "request.hpp"


namespace http::resource {
    /**
     * Абстрактный ресурс
     */
    class BasicResource {
    public:
        BasicResource(std::string  path);
        std::string path;

        virtual void handle(const server::Request &req, server::Reply &rep) = 0;

        virtual ~BasicResource() = default;
    };

    class StaticFileFactory {
        class StaticFileDef {
        public:
            StaticFileDef(const std::string& path, std::string webPath, server::mime_types::Mime type, bool allowCache = true);

            std::string webPath;
#ifdef USE_DEBUG
            std::string fsPath;
#endif
            server::mime_types::Mime type;
            bool allowCache;
            std::vector<char> content;

            ~StaticFileDef();
        };
        std::vector<StaticFileDef> files;
    public:
        StaticFileFactory();

        void registerFile(const std::string& path, const std::string &webPath, server::mime_types::Mime type, bool allowCache = true);

        void serve(const std::string& path, server::Reply& rep);

        ~StaticFileFactory();
    };

    using respGenerator = std::function<void(const server::Request &req, server::Reply &rep)>;

    /**
     * Класс ресурса для POST-запросов
     */
    class GenericResource final : public BasicResource {
    private:
        respGenerator generator_;

    public:
        GenericResource(const std::string& path, respGenerator  generator);

        void handle(const server::Request &req, server::Reply &rep) override;

        ~GenericResource() override;
    };

    void loadFile(const std::string& path, std::vector<char>& content);
}

#endif //RESOURCE_H
